import React, {useState, useEffect, useCallback} from 'react';
import {
	StyleSheet,
	ScrollView,
	Text,
	View,
	Button,
	ToastAndroid,
	PermissionsAndroid,
} from 'react-native';
import {useRoute} from '@react-navigation/native';
import prettyBytes from 'pretty-bytes';
import RNFS from 'react-native-fs';
import useS3 from '../hooks/useS3';

const FileDetails = () => {
	const route = useRoute();
	const s3 = useS3();
	const [data, setData] = useState(null);

	useEffect(() => {
		if (s3 && !data) {
			s3.headObject(
				{
					Bucket: route.params.bucket,
					Key: route.params.file,
				},
				(err, data) => {
					if (err) {
						ToastAndroid.show('Failed to fetch file info', ToastAndroid.SHORT);
						return;
					}

					setData(data);
				},
			);
		}
	}, [s3]);

	const download = useCallback(() => {
		PermissionsAndroid.request(
			PermissionsAndroid.PERMISSIONS.READ_EXTERNAL_STORAGE,
		)
			.then(granted => {
				if (granted != PermissionsAndroid.RESULTS.GRANTED) {
					throw new Error('No permission');
				}
			})
			.then(() =>
				PermissionsAndroid.request(
					PermissionsAndroid.PERMISSIONS.WRITE_EXTERNAL_STORAGE,
				),
			)
			.then(granted => {
				if (granted != PermissionsAndroid.RESULTS.GRANTED) {
					throw new Error('No permission');
				}
			})
			.then(() => {
				ToastAndroid.show('Downloading...', ToastAndroid.LONG);

				s3.getObject(
					{
						Bucket: route.params.bucket,
						Key: route.params.file,
					},
					(err, data) => {
						if (err) {
							ToastAndroid.show('Failed to download file', ToastAndroid.SHORT);
							return;
						}

						const dir = `${RNFS.DownloadDirectoryPath}/BrowseS3/${route.params.bucket}`;
						const file = `${dir}/${route.params.file.replace(/\//g, '_')}`;
						const fileContents = data.Body.toString('base64');

						RNFS.mkdir(dir)
							.then(() => RNFS.writeFile(file, fileContents, 'base64'))
							.then(() => RNFS.scanFile(file))
							.then(() => {
								ToastAndroid.show('Download completed', ToastAndroid.SHORT);
							})
							.catch(e => {
								ToastAndroid.show(
									'Failed to download file',
									ToastAndroid.SHORT,
								);
							});
					},
				);
			})
			.catch(() => {
				ToastAndroid.show(
					'You need to grant the write permission to download files',
					ToastAndroid.SHORT,
				);
			});
	}, [s3, route]);

	if (!data) {
		return <Text>Loading...</Text>;
	}

	return (
		<ScrollView style={styles.root}>
			<Text style={styles.bucket}>Bucket: {route.params.bucket}</Text>
			<Text style={styles.fileName}>Key: {route.params.file}</Text>
			<Text style={styles.info}>
				Last modified: {data.LastModified.toISOString()}
			</Text>
			<Text style={styles.info}>Size: {prettyBytes(data.ContentLength)}</Text>
			<Text style={styles.info}>Content type: {data.ContentType}</Text>
			<View style={styles.download}>
				<Button onPress={download} title="Download" />
			</View>
		</ScrollView>
	);
};

const styles = StyleSheet.create({
	root: {
		flex: 1,
		padding: 12,
	},
	bucket: {
		fontSize: 14,
		marginBottom: 12,
		color: '#000000',
	},
	fileName: {
		fontSize: 16,
		fontWeight: 'bold',
		marginBottom: 12,
		color: '#000000',
	},
	info: {
		fontSize: 14,
		marginBottom: 12,
	},
	download: {
		marginTop: 16,
	},
});

export default FileDetails;
